DynamicGenericInt

Struct DynamicGenericInt 

pub struct DynamicGenericInt<T: CiphertextOps, U: Sign> {
    pub bits: Vec<Arc<AtomicRefCell<T>>>,
    /* private fields */
}
A dynamic generic integer stored in unpacked form. A dynamic generic integer encrypts its bits in a few different ciphertexts of type T where the number of bits also represent the bit width

Fields

bits: Vec<Arc<AtomicRefCell<T>>>

The ciphertexts encrypting this dynamic generic integer’s bits in least-to-most significant order.

Implementations

Source

impl<T, U> DynamicGenericInt<T, U>
where T: CiphertextOps, U: Sign,

pub fn new(enc: &Encryption, n: usize) -> Self

Allocate a new DynamicGenericInt using trivial or precomputed (if T is L1GgswCiphertext) encryptions of zero.

pub fn from_bits_deep(bits: Vec<T>) -> Self

Create a DynamicGenericInt from a previously encrypted set of type T ciphertexts.

Remarks

bits are ordered from least to most significant.

This performs a deep copy of the underlying data.

pub fn from_bits_shallow(bits: Vec<Arc<AtomicRefCell<T>>>) -> Self

Create a DynamicGenericInt from The inner ref-counted set of T ciphertexts.

Remarks

bits are ordered from least to most significant.

This performs a shallow copy of the underlying data.

pub fn encrypt_secret( val: U::PlaintextType, enc: &Encryption, sk: &SecretKey, n: usize, ) -> Self

Encrypts the given integer.

Panics

If the given value is out of bounds.

pub fn decrypt(&self, enc: &Encryption, sk: &SecretKey) -> U::PlaintextType

Decrypts this encrypted integer and returns the contained integer message.

pub fn graph_inputs<'a>( &self, ctx: &'a FheCircuitCtx, ) -> DynamicGenericIntGraphNodes<'a, T, U>

Add input nodes to the given FheCircuitCtx.

pub fn with_decryption_fn<F>(&self, f: F) -> U::PlaintextType
where F: Fn(&T) -> bool,

Run a custom (e.g. threshold) decryption algorithm and return the result.

pub fn trivial( val: U::PlaintextType, enc: &Encryption, eval: &Evaluation, n: usize, ) -> Self

Create a trivial encryption of val.

Remarks

If T is L1GgswCiphertext, then the result will contain precomputed rather than trivial ciphertexts.

Panics

If val >= 2**n or val < -(2**n)

Source

impl<U> DynamicGenericInt<L1GlweCiphertext, U>
where U: Sign,

pub fn encrypt( val: U::PlaintextType, enc: &Encryption, pk: &PublicKey, n: usize, ) -> Self

Encrypt val using PublicKey.

Remarks

Requires the encryption parameters support RLWE encryption. crate::DEFAULT_128 have this property.

It’s generally recommended to encrypt a crate::fluent::PackedDynamicGenericInt and unpack the result, as they’re significantly smaller.