pub struct ComputeKeyNonFft {
pub bs_key: BootstrapKey<u64>,
pub ks_key: LweKeyswitchKey<u64>,
pub auto_key: AutomorphismKey<u64>,
pub ss_key: SchemeSwitchKey<u64>,
}Expand description
A set of keys that can be FFT’d and used during evaluation.
Remarks
Compute keys are quite large (100s of MB), so you should serialize with a protocol that can efficiently store arrays. Additionally, you should design your protocol around not having to frequently share these.
Fields
bs_key: BootstrapKey<u64>The bootstrapping key used internally in circuit bootstrapping operations.
ks_key: LweKeyswitchKey<u64>The keyswitch keys for converting L1 LWE ciphertexts to L0 LWE ciphertexts.
auto_key: AutomorphismKey<u64>An automorphism key used as part of circuit bootstrapping.
ss_key: SchemeSwitchKey<u64>Scheme switching keys used for turning L1 GLEV ciphertexts into L1 GGSW ciphertexts.
Implementations
Sourceimpl ComputeKeyNonFft
impl ComputeKeyNonFft
Sourcepub fn generate(secret_key: &SecretKey, params: &Params) -> Self
pub fn generate(secret_key: &SecretKey, params: &Params) -> Self
Generate the compute keys in non-fft form from the given secret keys.
Remarks
The params passed must be the same as those used during secret key generation.
Sourcepub fn fft(&self, params: &Params) -> ComputeKey
pub fn fft(&self, params: &Params) -> ComputeKey
Takes the fast-fourier transform of the keys, which is used during evaluation.
Sourcepub fn generate_with_default_params(secret_key: &SecretKey) -> Self
pub fn generate_with_default_params(secret_key: &SecretKey) -> Self
Generate the compute keys in non-fft form from the given secret keys with
the default parameters (crate::DEFAULT_128).
Remarks
The secret key must have also been generated with the default parameters.