pub struct DynamicGenericInt<T: CiphertextOps, U: Sign> {
pub bits: Vec<Arc<AtomicRefCell<T>>>,
/* private fields */
}Expand description
A dynamic generic integer stored in unpacked form. A dynamic generic integer encrypts its bits in
a few different ciphertexts of type T where the number of bits also represent the bit width
Fields
bits: Vec<Arc<AtomicRefCell<T>>>The ciphertexts encrypting this dynamic generic integer’s bits in least-to-most significant order.
Implementations
Sourceimpl<T, U> DynamicGenericInt<T, U>where
T: CiphertextOps,
U: Sign,
impl<T, U> DynamicGenericInt<T, U>where
T: CiphertextOps,
U: Sign,
Sourcepub fn new(enc: &Encryption, n: usize) -> Self
pub fn new(enc: &Encryption, n: usize) -> Self
Allocate a new DynamicGenericInt using trivial or precomputed (if T is L1GgswCiphertext) encryptions
of zero.
Sourcepub fn from_bits_deep(bits: Vec<T>) -> Self
pub fn from_bits_deep(bits: Vec<T>) -> Self
Create a DynamicGenericInt from a previously encrypted set of type T ciphertexts.
Remarks
bits are ordered from least to most significant.
This performs a deep copy of the underlying data.
Sourcepub fn from_bits_shallow(bits: Vec<Arc<AtomicRefCell<T>>>) -> Self
pub fn from_bits_shallow(bits: Vec<Arc<AtomicRefCell<T>>>) -> Self
Create a DynamicGenericInt from The inner ref-counted set of T ciphertexts.
Remarks
bits are ordered from least to most significant.
This performs a shallow copy of the underlying data.
Sourcepub fn encrypt_secret(
val: U::PlaintextType,
enc: &Encryption,
sk: &SecretKey,
n: usize,
) -> Self
pub fn encrypt_secret( val: U::PlaintextType, enc: &Encryption, sk: &SecretKey, n: usize, ) -> Self
Sourcepub fn decrypt(&self, enc: &Encryption, sk: &SecretKey) -> U::PlaintextType
pub fn decrypt(&self, enc: &Encryption, sk: &SecretKey) -> U::PlaintextType
Decrypts this encrypted integer and returns the contained integer message.
Sourcepub fn graph_inputs<'a>(
&self,
ctx: &'a FheCircuitCtx,
) -> DynamicGenericIntGraphNodes<'a, T, U>
pub fn graph_inputs<'a>( &self, ctx: &'a FheCircuitCtx, ) -> DynamicGenericIntGraphNodes<'a, T, U>
Add input nodes to the given FheCircuitCtx.
Sourcepub fn with_decryption_fn<F>(&self, f: F) -> U::PlaintextType
pub fn with_decryption_fn<F>(&self, f: F) -> U::PlaintextType
Run a custom (e.g. threshold) decryption algorithm and return the result.
Sourcepub fn trivial(
val: U::PlaintextType,
enc: &Encryption,
eval: &Evaluation,
n: usize,
) -> Self
pub fn trivial( val: U::PlaintextType, enc: &Encryption, eval: &Evaluation, n: usize, ) -> Self
Create a trivial encryption of val.
Remarks
If T is L1GgswCiphertext, then the result
will contain precomputed rather than trivial ciphertexts.
Panics
If val >= 2**n or val < -(2**n)
Sourceimpl<U> DynamicGenericInt<L1GlweCiphertext, U>where
U: Sign,
impl<U> DynamicGenericInt<L1GlweCiphertext, U>where
U: Sign,
Sourcepub fn encrypt(
val: U::PlaintextType,
enc: &Encryption,
pk: &PublicKey,
n: usize,
) -> Self
pub fn encrypt( val: U::PlaintextType, enc: &Encryption, pk: &PublicKey, n: usize, ) -> Self
Encrypt val using PublicKey.
Remarks
Requires the encryption parameters support RLWE encryption. crate::DEFAULT_128
have this property.
It’s generally recommended to encrypt a crate::fluent::PackedDynamicGenericInt
and unpack the result, as they’re significantly smaller.